Learn R Programming

geommc (version 1.3.1)

geommc-package: geommc: Geometric Markov Chain Sampling

Description

Simulates from discrete and continuous target distributions using geometric Metropolis-Hastings (MH) algorithms. Users specify the target distribution by an R function that evaluates the log un-normalized pdf or pmf. The package also contains a function implementing a specific geometric MH algorithm for performing high-dimensional Bayesian variable selection.

Arguments

Author

Maintainer: Vivekananda Roy vroy@iastate.edu (ORCID)

See Also